package cz.upol.pja.lecture01;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

// pouzije se mapovani ve web.xml
//@WebServlet(name = "fractal", urlPatterns = {"/fractal"})
public class FractalServlet extends HttpServlet {

    private static final int WIDTH = 400;
    private static final int HEIGHT = 400;
    
    private static final double SIZE_REDUCTION = 0.8;
    private static final int ANGLE_COEF = 30;
 
    
    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        
	// format vystupu
        resp.setContentType("image/png");
	
	// z konfigurace servletu (viz web.xml) si vytahnu pocet iteraci
        String iterations = getServletConfig().getInitParameter("iterations");
        int iters = 10;
        if ((iterations != null) && (!iterations.equals(""))) 
            iters = Integer.parseInt(iterations);
        
	// vykresleni
        BufferedImage bi = new BufferedImage(WIDTH, HEIGHT, BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = bi.createGraphics();
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        
        g.setPaint(Color.BLACK);
        
        drawTree(g, new Point(WIDTH / 2, HEIGHT), 70, 90, iters);
	
	// provedeni samotneho zapisu
        ImageIO.write(bi, "PNG", resp.getOutputStream());
    }
    
    private static void drawTree(Graphics2D g, Point startPos, int size, int angle, int iteration) {
        if (iteration == 0) return;
 
        Point endPos = getLocation(startPos, size, angle);
        g.drawLine(startPos.x, startPos.y, endPos.x, endPos.y);
	
        drawTree(g, endPos, (int) (size * SIZE_REDUCTION), angle + ANGLE_COEF, iteration - 1);
        drawTree(g, endPos, (int) (size * SIZE_REDUCTION), angle - ANGLE_COEF, iteration - 1);
    }
    
    /**
     * pro dany bod p vraci souradnici ve vzdalenosti distance pod uhlem angle
     */
    private static Point getLocation(Point p, int distance, int angle) {
	double rangle = -Math.toRadians(angle); // minus, aby se to chovalo jako v prvnim kvadrantu
        int dx = (int) (p.x + distance * Math.cos(rangle));
        int dy = (int) (p.y + distance * Math.sin(rangle));
        return new Point(dx, dy);
    }

}
