
package cz.upol.pja.lecture01;

import java.io.Serializable;
import java.util.Random;
import java.util.List;
import java.util.ArrayList;
import java.util.Collections;

/**
 * Logika hry hadani cisel
 */
public class GuessNumber implements Serializable {
    
    private final int secretValue;
    private final List<Integer> history;
    private GameStatus status;
    
    public GuessNumber() {
	secretValue = (new Random()).nextInt(1000);
	history = new ArrayList<>();
	status = GameStatus.IN_PROGRESS;
    }
    
    /**
     * Pokus o uhadnuti cisla.
     * 
     * Vraci kladnou hodnotu, pokud bylo hadane cislo vetsi nez tajne,
     * pokud bylo uhadnuto vraci 0, jinak vraci zapornou hodnotu.
     *
     * Metoda meni nastaveni atributu {@link GuessNumber#status},
     * pokud se uzivatel trefil nebo pokud vycerpal vsechny sve pokusy.
     */
    public int guess(int value) {
        if (history.size() > 10) status = GameStatus.GAME_OVER;
        if (value == secretValue) status = GameStatus.WON;
        history.add(value);
        return value - secretValue;
    }
    
    public List<Integer> getHistory() {
        return Collections.unmodifiableList(history);
    }
    
    public GameStatus getStatus() {
        return status;
    }
    
    public static enum GameStatus {
        IN_PROGRESS,
        GAME_OVER,
        WON
    }
    
}
